#!/usr/bin/perl 

#next_db_link.cgi -Look up a gene in the NEXT_DB in situ DB.
#
#---------------------------
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#---------------------------
#
#Version: 0.001
#
#Last modified: 7/02
#Written by Jim Lund in the lab of Stuart Kim, Stanford University.
#jiml@stanford.edu
#
#Postal address: Department of Developmental Biology, B365
#		 279 Campus Dr.
#                Stanford, CA 94305
#
#Web site: http://worm-chip.stanford.edu
#---------------------------
#HISTORY
#v.001, 7/5/02
#-Split off from show_gene.cgi.
#



#
#Import modules used.
#
use LWP::UserAgent;
use HTTP::Request::Common qw(POST);
use CGI qw(:standard);


#
#Subroutines
#
sub PARSE_FORM;
sub SELECT_GENES;
sub SELECT_GENES2;
sub PRINT_OUTPUT;
sub PRINT_HTML;
sub COMBINE_LISTS;
sub PRINT_TEXT;
sub FIND_PLATE;
sub NEXT_DB_LINK;
sub POST_FORM;
sub GET_ROW;
sub GET_WEB_PAGE;



#
#Global variables.
#

#
#NEXT_DB in situ images URL.
#
my $in_situ_base_url="http://nematode.lab.nig.ac.jp/cgi-bin/db/ShowGeneInfo.sh";


#DEBUG
#$|=1;
#print header();
#exit;


#
#Get query gene list from form.
#
$query_gene=&PARSE_FORM;


#
#Look up gene at NEXT_DB site.
#
@in_situ_group=&NEXT_DB_LINK($query_gene);


#
#If gene not found.
#
if ($#in_situ_group == -1) {
    &PRINT_FAIL;
    exit;
    }


#
#Look up EST clusters at NEXT_DB site, to find which one has the images.
#
foreach $group (@in_situ_group) {

#
#Look up the EST group page.
#
    $html_results=&GET_WEB_PAGE("$in_situ_base_url?celk=$group");

#
#Check if it has images.
#
    if ($html_results !~ /no image/i) { push(@image_group,$group); }
    }


if ($#image_group == -1) { &PRINT_FAIL; }
elsif ($#image_group == 0) {
    $page_result= new CGI;
    print $page_result->redirect($in_situ_base_url.'?celk='.$image_group[0]);
    }
else {
#
#Print the multiple matching EST clusters.
#
    &PRINT_GROUPS(@image_group);
    }


exit;
#
#End of main section.
#


#
#Subroutines
#



sub PRINT_GROUPS {
@image_group=@_;

print header();
print <<EOM;
<HTML>
<HEAD>
<TITLE>NEXT DB lookup for $query_gene</TITLE>
</HEAD>
<BODY>
<H1> NEXT DB lookup for $query_gene </H1>
<P>
EOM

foreach $group (@image_group) {
    print"<P>NEXT DB EST cluster <A HREF='$in_situ_base_url?celk=$group'>$group</A>\n";
    }
print "</BODY> </HTML>\n";
}


sub PRINT_FAIL {

print header();
print <<EOM;
<HTML>
<HEAD>
<TITLE>NEXT DB lookup for $query_gene</TITLE>
</HEAD>
<BODY>
<H1> NEXT DB lookup for $query_gene </H1>
<P>
Gene not found in NEXT DB in situ image database.
</BODY> </HTML>
EOM
}




sub NEXT_DB_LINK {
my ($gene)=@_;

my ($html_results,@groups,%groups,$row,$i,$link_row);

my $in_situ_url="http://nematode.lab.nig.ac.jp/cgi-bin/db/Keysrch.sh";

my (%form_params);


#
#Search by formal name or gene name.
#
if ($gene =~ /-/) { %form_params=("gene",$gene); }
else { %form_params=("cds",$gene); }

print STDERR "G:$gene\n";
foreach $key (keys %form_params) { print STDERR "FP: $key,$form_params{$key}\n";}
#
#Do the search on the gene name.
#
$html_results=&POST_FORM($in_situ_url,%form_params);


#
#If no gene found, return.
#
if ($html_results=~/There is not a hit/i) { return; }

$html_results=~s/^.+?<table>/is;
$html_results=~s/<\/table>.+$/is;


#
#Find the 'group' row.  This holds the link to the images for the gene.
#
($row,$html_results)=&GET_ROW($html_results);
for ($i=0;$i <= $#{$row};$i++) {
    if ($$row[$i]=~/^\s*group\s*$/i) {
	$link_row=$i;
	last;
	}
    }


#
#Now examine all the rows.
#
while(1) {
    undef $row;
    ($row,$html_results)=&GET_ROW($html_results);
    if ($#{$row} == -1) { last; }
    $$row[$link_row]=~s/<[^>]*>//sg;
    $$row[$link_row]=~s/(^\s*|\s*$)//sg;
    $groups{$$row[$link_row]}=1;
    }

@groups=keys %groups;

print STDERR "GROUPS: @groups\n";

return(@groups);
}



sub POST_FORM {
my ($url,%form_params)=@_;

my ($ua,$req,$html_returned);
my $web_time_out=60*30;

$ua = new LWP::UserAgent;

$req = POST $url, \%form_params;

$ua->timeout($web_time_out);
$html_returned= $ua->request($req)->as_string;

return($html_returned);
}



#
#Extract the first HTML row in the provided code.
#
sub GET_ROW {
my ($html)=@_;

my (@row,$row);

$html=~s/^.*?<TR[^>]*>//is;
$html=~s/^(.*?)<\/TR[^>]*>//is;
$row=$1;

while ($row=~s/^.*?<T[HD][^>]*>(.*?)<\/T[HD][^>]*>//si) {
    push(@row,$1);
    }

return(\@row,$html);
}


############################################################################
#Called by: main
#Calls: none
#
#Get the list of genes from the form.
#
sub PARSE_FORM {
my ($form_result,$query);

$form_result= new CGI;

$query= $form_result->param('gene');

return ($query);
}


sub GET_WEB_PAGE {
my ($url)=@_;
my $web_time_out=60*30;

$ua = new LWP::UserAgent;
my $req = new HTTP::Request('GET',$url);

$ua->timeout($web_time_out);
$html_returned= $ua->request($req)->content;

return($html_returned);
}


#
#End of program.



