#!/usr/bin/perl

#show_gene.cgi -Provide gene info based on web query.
#
#This script provides access to my Worm DB and to Acedb.  Genes are entered,
#and the database info for the gene is displayed.
#
#---------------------------
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#---------------------------
#
#Version: 0.010
#
#Last modified: 10-2005
#
#Written by Jim Lund, Department of Biology, University of Kentucky.
#jiml at uky dot edu
#Web site: http://elegans.uky.edu
#
#---------------------------
#HISTORY
#v.010, 10-05
#-Update to work with new database structure (primer table to sjj_primer).
#
#v.005, 3-26-03
#-Update links to Wormbase.  Split left side menu into a separate file,
# 'Menu.left'.  Update menu links.
#-If given hidden option 'all_genes', like this 'show_gene.cgi?all_genes=1&text_only=1'
# all genes are printed out.
#
#v.004, 7-11-02
#-Move to workhorse.  Add generalized Db connection code.  Add NEXT_DB links.
# Fix HTML format.  Drop 'show cDNA' button from the associated HTML form.
#
#v.003, 3-22-02
#-Added some 'my's, now pass some parameters.  This allows this to work
# with mod_perl.
#
#v.002, 9-17-01
#-Incease total upload size to 10Mb from 1Mb, it was having trouble with large
# lists of genes.
#

use CGI qw(:standard);
use DBI;
use WebConfig;
use Worm;


#
#Subroutines
#
sub PARSE_FORM;
sub SELECT_GENES;
sub PRINT_OUTPUT;
sub PRINT_TEXT_ONLY;
sub NEXT_DB_LINK;
sub POST_FORM;
sub GET_ROW;


#
#Send out output without delay.
#
$|=1;


#
#Global variables.
#
my ($text_only,$smd_result,$all_genes);
my ($number_submitted,$search_rows_ref,$table_rows_ref,$columns);
my $number_submitted=0;
my $not_found_ref;
my @query_genes=();


#
#Setup local config variables.
#
BEGIN {
  WebConfig::CGI_init;
}


#
#Initailize db connection.
#
my ($dbh,$DB_DATE) = WebConfig::DB_init;


#
#Get query gene list from form.
#
($text_only,$smd_result,$all_genes,@query_genes)=&PARSE_FORM;
$number_submitted=$#query_genes+1;


#
#Search the database for the genes.
#
($search_rows_ref,$table_rows_ref,$not_found_ref,$columns)=&SELECT_GENES(@query_genes);



#
#Print the output of the search.
#
if ($text_only) {&PRINT_TEXT_ONLY($search_rows_ref,$table_rows_ref,$columns,$smd_result,$number_submitted,$not_found_ref);}
else {&PRINT_OUTPUT($search_rows_ref,$table_rows_ref,$columns,$smd_result,$number_submitted,$not_found_ref);}


exit;
#
#End of main section.
#


#
#Subroutines
#


#
#Get the list of genes from the form.
#
sub PARSE_FORM {

my ($form_result,@query);
my ($text_only,$smd_ready,$all_genes);
my (@gene_lines,$line,$buffer,$file_handle,$max_file_size,$bytes_read);

$form_result= new CGI;

$query[0]= $form_result->param('Gene');
$query[0]=~s/^\s+//;
@query=split(/\s+/,$query[0]);

if (!$query[0]) {
    undef @query;
    @gene_lines=split(/\s*[\015\012]+\s*/,$form_result->param('Gene_list'));
    foreach $line (@gene_lines) {
	if ($line=~/^\s*(\S+)/) {
	    $query[$#query+1]=$1;
	    $query[$#query]=~ s/^"\s*//;
	    $query[$#query]=~ s/\s*"$//;
	    if (!$query[$#query]) {pop @query;}
	    }
	}
    }

if (!$query[0]) {
    undef @query;
    undef $buffer;
    $max_file_size=10000000;
    $file_handle=$form_result->param('Gene_file');
    while ($bytes = read($file_handle,$buffer,1024)) {
	$genes_file.=$buffer;
	$bytes_read+=$bytes;
	if ($bytes_read > $max_file_size) {last;}
	}
    @gene_lines=split(/\s*[\015\012]+\s*/,$genes_file);
    foreach $line (@gene_lines) {
	if ($line=~/^\s*(\S+)/) {
	    $query[$#query+1]=$1;
	    $query[$#query]=~ s/^"\s*//;
	    $query[$#query]=~ s/\s*"$//;
	    if (!$query[$#query]) {pop @query;}
	    }
	}
    }

#
#Make gene names upper case.
#
for ($i=0;$i < $#query+1;$i++) { $query[$i]=uc($query[$i]); }


$text_only = $form_result->param('text_only');
$smd_ready = $form_result->param('smd_ready');
$all_genes = $form_result->param('all_genes');

return ($text_only,$smd_ready,$all_genes,@query);
}



#
#Do the search.
#Ruturn the number of cols in the result rows, the row headers, and
#the data rows.
#
sub SELECT_GENES {
my (@query_genes)=@_;

my ($sth,@row,@table_rows,$gene,@search_rows);
my ($cols,$cols_this_row,@not_found);
my ($gene,$result_rows,%gene_name_key,$gene_name_key);

my $search1a="SELECT gene_name_key FROM gene_all_names WHERE gene_name=?";
my $search1b="SELECT gene_name_key FROM gene_all_names";

my $search2="SELECT gene_cos_name,all_names,proteome_descr,wormbase_descr,wormbase_cds FROM gene g, gene_descr d WHERE g.gene_name_key=d.gene_name_key AND g.gene_name_key=?";
my $search3="SELECT * FROM genes_map WHERE gene=%s ORDER BY product_length";
my $search4="SELECT * FROM genes_map,sjj_primers WHERE genes_map.gene_rough_name=primers.gene AND (genes_map.gene=%s OR genes_map.gene_rough_name=%s)";

$dup_genes=0;
$dup_names=0;


#
#Get gene_name_keys.
#
if ($all_genes) {

    $sth= $dbh->prepare($search1b);
    $sth->execute();
    my $result_rows=0;
    while ( @row = $sth->fetchrow_array ) {
        if (exists($gene_name_key{$row[0]})) {$dup_genes++;}
    	$gene_name_key{$row[0]}=1;
    	$result_rows++;
    	} 

    }else{

    $sth= $dbh->prepare($search1a);
    foreach my $gene (@query_genes) {
        $sth->execute($gene);
        my $result_rows=0;
        while ( @row = $sth->fetchrow_array ) {
    	    if (exists($gene_name_key{$row[0]})) {$dup_genes++;}
      	    $gene_name_key{$row[0]}=1;
    	    $result_rows++;
    	    } 
        if ($result_rows > 1) {$dup_names++;}
        elsif ($result_rows == 0) {push(@not_found,$gene);}
    }

 }



#
#Do real search.
#
$sth= $dbh->prepare($search2);
foreach $gene_name_key (keys %gene_name_key) {
    $sth->execute($gene_name_key);

#
#Get row labels and number of resulting columns.
#
    $cols_this_row=$sth->{NUM_OF_FIELDS};
    if ($cols_this_row > $cols) {
	$cols=$cols_this_row;
   	@table_rows=@{$sth->{NAME}};
#Column wormbase_cds gets combined with wormbase_descr.
	pop @table_rows;
	}

    $rows_returned=0;
    while ( @row = $sth->fetchrow_array ) {
	$rows_returned++;

	my $wb_descr1 = pop @row;
	my $wb_descr2 = pop @row;
	my $wb_descr = $wb_descr1.'; '.$wb_descr2;
	$wb_descr =~ s/(^; |; $)//g;
	push (@row,$wb_descr);

        push(@search_rows,[@row]);
        }
    if (!$rows_returned && !$all_genes) {
	push(@search_rows,['Not cloned',$gene]);
	}
    }

#
#Done with datbase.
#
$dbh->disconnect;


return (\@search_rows,\@table_rows,\@not_found,$cols);
}





#
#Format and print the output of the search.
#The table labels are in an array ref, $table_rows_ref.
#The number of columns is in $cols.
#The rows are in a ref to an array of refs, $search_rows_ref.
#
sub PRINT_TEXT_ONLY {
my ($search_rows_ref,$table_rows_ref,$cols,$smd_result,$number_submitted,$not_found_ref)=@_;

my ($row_ref,$gene_rough_name_column,$gene_column);
my $gene_name_col=0;

#
#HTML header
#
print header(-type=>'text/plain');


#
#Print search summary stats.
#
print"$number_submitted gene names submitted.\n";
print $#{$search_rows_ref}+1," genes found in database.\n\n";
if ($#{$not_found_ref} >= 0) {
    print"Genes not found in the database:\n";
    print "@{$not_found_ref}\n\n";
    }
if ($dup_names) {
    print"$dup_names gene name(s) mapped to more than one gene in the database.\n";
    }
if ($dup_genes) {
    print"$dup_genes entered gene name(s) mapped to the identical gene(s) in the database.\n\n";
    }


#
#Show SMD '::' separated gene list.
#
if ($smd_result) {
    foreach $row_ref (@{$search_rows_ref}) {
        $smd_genes=join('::',$smd_genes,$$row_ref[$gene_name_col]);
	}
    $smd_genes=~s/^\:\://;
    print <<EOM;

Genes are shown below as a :: separated list of genes for use in SMD clustering:

$smd_genes

EOM
    }


#
#Info for making this into a SMD genelist.
#
print <<EOM;
The output columns are tab-delimitted.  To convert this file into a SMD 
genelist, delete the lines before and including the '------' line.

EOM


#
#Print table header.
#
print"C. elegans genes\n\n";


#
#Print result field names.  Also find the column which has the gene name.
#This column will be linked to proteome.
#
for ($field=0;$field <= $#{$table_rows_ref};$field++) {
    if ($field) {print"\t";}
    print $$table_rows_ref[$field];
    }
print"\n";
print"-"x78,"\n";


#
#Print rows.
#
foreach $row_ref (sort {$$a[0] cmp $$b[0]} @{$search_rows_ref}) {
    for ($i=0;$i< $cols;$i++) {
	if ($i) {print"\t";}
        $datum=$$row_ref[$i];
    	chomp $datum;
        print $datum;
        }
    print"\n";
    }
}





#
#Format and print the output of the search.
#The table labels are in an array ref, $table_rows_ref.
#The number of columns is in $cols.
#The rows are in a ref to an array of refs, $search_rows_ref.
#
sub PRINT_OUTPUT {
my ($search_rows_ref,$table_rows_ref,$cols,$smd_result,$number_submitted,$not_found_ref)=@_;

my ($row_ref,$gene_rough_name_column,$gene_column);
my $gene_name_col=0;


#
#NEXT_DB in situ images URL.
#
my $in_situ_base_url="next_db_link.cgi";
    


#
#HTML header
#
print header();
my $HTML_results= new CGI;


print <<EOM;
<HTML>
<HEAD>
<link rel="icon" href="../../favicon.ico" type="image/x-icon">
<link REL="shortcut icon" HREF="../../favicon.ico" TYPE="image/x-icon">
<link rel="stylesheet" title="Worm (Default)" href="../../worm-bg.css" type="text/css" media="screen">
<TITLE> Results from comparison of the two gene lists </TITLE>
</HEAD>

<BODY>

<!--#include virtual="/links.php" -->

<div id="content">

EOM

print $HTML_results->center(h1('Worm gene database search results')),"<P><P>\n";



#
#Print search summary stats.
#
print"$number_submitted gene names submitted.<BR>\n";
print $#{$search_rows_ref}+1," genes found in database.<P>\n";
if ($#{$not_found_ref} >= 0) {
    print"Genes not found in the database:<BR>\n";
    print "@{$not_found_ref}<P>\n";
    }
if ($dup_names) {
    print"$dup_names gene name(s) mapped to more than one gene in the database.<P>\n";
    }
if ($dup_genes) {
    print"$dup_genes entered gene name(s) mapped to the identical gene(s) in the database.<P>\n";
    }



#
#Show SMD '::' separated gene list.
#
if ($smd_result) {
    foreach $row_ref (@{$search_rows_ref}) {
        $smd_genes=join('::',$smd_genes,$$row_ref[$gene_name_col]);
	}
    $smd_genes=~s/^\:\://;
    print <<EOM;
Genes are shown below as a :: separated list of genes for use in SMD clustering:
<P>
$smd_genes
<P>

EOM
    }



#
#If there are no genes to print, print HTML tail and return.
#
if ($#{$search_rows_ref} < 0) {
    print <<EOM;

</TD></TR>
</TABLE>

</TD></TR>
</TABLE>

</BODY>
</HTML>
EOM
    }


#
#Print table header.
#
printf("<TABLE COLS=%s BORDER=1 CELLSPACING=0 CELLPADDING=1 WIDTH='100%'>\n",$cols);
print"<CAPTION> <H2> <I>C. elegans</I> genes </H2> </CAPTION>\n";


#
#Add Proteome link and ACEDB link columns to table header row.
#
unshift(@{$table_rows_ref},'Proteome link');
$proteome_col=2;
unshift(@{$table_rows_ref},'WormBase link');
$ACEDB_col=1;
unshift(@{$table_rows_ref},'NEXT_DB in situ images');
$in_situ_col=0;


#
#Print result field names.  Also find the column which has the gene name.
#This column will be linked to proteome.
#
print"<TR>\n";
for ($field=0;$field <= $#{$table_rows_ref};$field++) {
    my $label = $$table_rows_ref[$field];
    if ($label eq 'gene_cos_name') { $label = 'Gene sequence name'; }
    if ($label eq 'all_names') { $label = 'All names'; }
    if ($label eq 'proteome_descr') { $label = 'Proteome description'; }
    if ($label eq 'wormbase_descr') { $label = 'WormBase description'; }
    print "<TH> $label </TH>\n";

    if ($$table_rows_ref[$field] eq "all_names") { $all_names_column = $field; }

    if ((!$gene_column) && ($$table_rows_ref[$field] eq "gene_cos_name")) {
        $gene_column=$field;
        }

    if ((!$gene_rough_name_column)
                &&
        ($$table_rows_ref[$field] eq "gene_cos_name")) {
        $gene_rough_name_column=$field;
        }
    }
print"</TR>\n";


#
#Print rows.
#
foreach $row_ref (sort {$$a[0] cmp $$b[0]} @{$search_rows_ref}) {
    print"<TR ALIGN=CENTER> ";


#
#Find which cell to link to the NEXT_DB link.
#
    if ($$row_ref[$gene_rough_name_column-3]) {
        $in_situ_url=$in_situ_base_url;
        $datum=$$row_ref[$gene_rough_name_column-3];
        }
    elsif ($$row_ref[$gene_rough_name_column-5]) {
        $in_situ_url=$in_situ_base_url;
        $datum=$$row_ref[$gene_rough_name_column-5];
        }
    else { $in_situ_url=''; }
    if ($in_situ_url) {
        $in_situ_link="<A HREF='$in_situ_base_url?gene=$datum'>N</A>";
        printf("<TD> %s </TD>\n",$in_situ_link);
        }
    else {print"<TD> &nbsp; </TD>\n";}



#
#Find which cell to link to ACEDB, and build the url.
#
    undef $datum;
    if ($$row_ref[$gene_column-2] !~ /NULL/i) {
        $datum=$$row_ref[$gene_column-2];
        }
    elsif ($$row_ref[$gene_rough_name_column-2] !~ /Not/i) {
        $datum=$$row_ref[$gene_rough_name_column-2];
        }

    if ($datum) {
	my ($ACE_url) = Worm::Wormbase_url($datum,undef,undef,'W');
        printf("<TD> %s </TD>\n",$ACE_url);
        }
    else { print"<TD> &nbsp; </TD>\n"; }

#
#Find which cell to link to proteome, and build the url.
#
    undef $datum;
    if ($$row_ref[$gene_rough_name_column-2] !~ /Not/i) {
        if ($$row_ref[$gene_column-2] !~ /NULL/i) {
            $datum=$$row_ref[$gene_column-2];
            }
        else {
            $datum=$$row_ref[$gene_rough_name_column-2];
            }

	my ($proteome_url) = Worm::Proteome_url($datum,undef,'P');
        printf("<TD> %s </TD>\n",$proteome_url);

        }
    else { print"<TD> &nbsp; </TD>\n"; }


#
#Print data.
#
    for ($i=0;$i< $cols;$i++) {
        if (!($datum=$$row_ref[$i])) {$datum="&nbsp;";}
        if ($i == $all_names_column) { $datum =~ s/;/; /g; }
        printf("<TD> %s </TD>\n",$datum);
	}
    }



#
#Close table.
#
print"</TABLE>\n";


#
#HTML tail. Close enclosing tables.
#
print <<EOM;
<P>

<a href="show_gene_prog.html">View the program.</a>
            
<!--#include virtual="/tail.php" -->

        </div>

</BODY>
</HTML>
EOM
}



sub NEXT_DB_LINK {
my ($gene)=@_;

my ($html_results,@groups,%groups,$row,$i,$link_row);

my $in_situ_url="http://nematode.lab.nig.ac.jp/cgi-bin/db/Keysrch.sh";

my %form_params=("cds",$gene);

#
#Do the search on the gene name.
#
$html_results=&POST_FORM($in_situ_url,%form_params);


#
#If no gene found, return.
#
if ($html_results=~/There is not a hit/i) { return; }

$html_results=~s/^.+?<table>/is;
$html_results=~s/<\/table>.+$/is;


#
#Find the 'group' row.  This holds the link to the images for the gene.
#
($row,$html_results)=&GET_ROW($html_results);
for ($i=0;$i <= $#{$row};$i++) {
    if ($$row[$i]=~/^\s*group\s*$/i) {
        $link_row=$i;
        last;
        }
    }



#
#Now examine all the rows.
#
while(1) {
    undef $row;
    ($row,$html_results)=&GET_ROW($html_results);
    if ($#{$row} == -1) { last; }
    $$row[$link_row]=~s/<[^>]*>//sg;
    $$row[$link_row]=~s/(^\s*|\s*$)//sg;
    $groups{$$row[$link_row]}=1;
    }

@groups=keys %groups;

return(@groups);
}




sub POST_FORM {
my ($url,%form_params)=@_;

my ($ua,$req,$html_returned);
my $web_time_out=60*30;

$ua = new LWP::UserAgent;

$req = POST $url, \%form_params;

$ua->timeout($web_time_out);
$html_returned= $ua->request($req)->as_string;

return($html_returned);
}



#
#Extract the first HTML row in the provided code.
#
sub GET_ROW {
my ($html)=@_;

my (@row,$row);

$html=~s/^.*?<TR[^>]*>//is;
$html=~s/^(.*?)<\/TR[^>]*>//is;
$row=$1;

while ($row=~s/^.*?<T[HD][^>]*>(.*?)<\/T[HD][^>]*>//si) {
    push(@row,$1);
    }

return(\@row,$html);
}




#
#End of program.




